@@ -16,10 +16,6 @@
 /*                                                               */
 /* ============================================================= */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #ifdef _OPENMP
 #include <omp.h>
 #endif
@@ -33,7 +29,6 @@
 #define streamDefTimestep cdo_def_timestep
 
 #include "afterburner.h"
-#include "constants.h"
 #include "compare.h"
 #include "cdo_options.h"
 #include "cdi_lockedIO.h"
@@ -54,15 +49,13 @@ struct RARG
   AfterControl *globs;
 };
 
-cdo::Task *afterReadTask = nullptr;
-
 static bool lstdout = true;
 static int Source = 0;
 static int ofiletype = -1;
 static int DataType = -1;
 
 static char *filename;
-static const char **ifiles;
+static std::vector<const char *> ifiles;
 static char *ifile = nullptr;
 
 static int ofileidx = 0;
@@ -71,11 +64,11 @@ static int specGridID = -1;
 static int gaussGridID = -1;
 static int iVertID = -1;
 static int oVertID = -1;
+static int oVertID_half = -1;
 
 static bool Lhybrid2pressure = false;
 
 static int TsID;
-bool afterReadAsync = true;
 
 #define TIMESTEP_INTERVAL -1
 #define MONTHLY_INTERVAL 0
@@ -112,7 +105,7 @@ FreeMean(struct Variable *vars)
   for (int code = 0; code < MaxCodes; ++code)
     if (vars[code].mean)
       {
-        Free(vars[code].mean);
+        free(vars[code].mean);
         vars[code].mean = nullptr;
       }
 }
@@ -282,7 +275,7 @@ after_SwitchFile(AfterControl *globs)
 }
 
 static CdiDateTime
-after_getDateTime(struct Date datetime)
+after_getDateTime(const struct Date &datetime)
 {
   CdiDateTime cdiDateTime{};
   cdiDateTime.date = cdiDate_encode(datetime.yr, datetime.mo, datetime.dy);
@@ -376,8 +369,8 @@ static int num_recs = 0;
 static void *
 after_readTimestep(void *arg)
 {
-  int varID, gridID, zaxisID, levelID, timeID;
-  size_t nmiss;
+  int varID, gridID, zaxisID, levelID, timeType;
+  size_t numMissVals;
   auto rarg = (RARG *) arg;
 
   auto nrecs = rarg->nrecs;
@@ -385,7 +378,7 @@ after_readTimestep(void *arg)
   auto vars = rarg->vars;
   auto globs = rarg->globs;
 
-  for (int code = 0; code < MaxCodes; ++code) vars[code].nmiss0 = 0;
+  for (int code = 0; code < MaxCodes; ++code) vars[code].numMissVals0 = 0;
 
   for (int recID = 0; recID < nrecs; ++recID)
     {
@@ -397,7 +390,7 @@ after_readTimestep(void *arg)
       // Skip records containing unneeded codes
       if (!vars[code].needed0) continue;
 
-      vlistInqVar(globs->ivlistID, varID, &gridID, &zaxisID, &timeID);
+      vlistInqVar(globs->ivlistID, varID, &gridID, &zaxisID, &timeType);
 
       auto leveltype = zaxisInqType(zaxisID);
 
@@ -409,7 +402,7 @@ after_readTimestep(void *arg)
           auto level = (int) zaxisInqLevel(zaxisID, levelID);
           for (int i = 0; i < globs->NumLevelRequest; ++i)
             {
-              if (IS_EQUAL(globs->LevelRequest[i], level))
+              if (is_equal(globs->LevelRequest[i], level))
                 {
                   levelOffset = i;
                   break;
@@ -433,14 +426,14 @@ after_readTimestep(void *arg)
 
       if (analysisData)
         {
-          streamReadRecord(globs->istreamID, globs->Field, &nmiss);
-          after_AnalysisAddRecord(globs, vars, code, gridID, zaxisID, levelID, nmiss);
+          streamReadRecord(globs->istreamID, globs->varray.data(), &numMissVals);
+          after_AnalysisAddRecord(globs, vars, code, gridID, zaxisID, levelID, numMissVals);
         }
       else
         {
           double *dataptr = after_get_dataptr(vars, code, gridID, zaxisID, levelID);
-          streamReadRecord(globs->istreamID, dataptr, &nmiss);
-          after_EchamAddRecord(globs, vars, code, gridID, zaxisID, levelID, nmiss);
+          streamReadRecord(globs->istreamID, dataptr, &numMissVals);
+          after_EchamAddRecord(globs, vars, code, gridID, zaxisID, levelID, numMissVals);
         }
 
       if (iVertID != -1 && oVertID != -1 && (vars[code].izaxisID == iVertID)) vars[code].ozaxisID = oVertID;
@@ -499,7 +492,7 @@ after_setEndOfInterval(AfterControl &globs, int nrecs)
 static void
 after_moveTimestep(struct Variable *vars)
 {
-  for (int code = 0; code < MaxCodes; ++code) vars[code].nmiss = vars[code].nmiss0;
+  for (int code = 0; code < MaxCodes; ++code) vars[code].numMissVals = vars[code].numMissVals0;
 
   for (int code = 0; code < MaxCodes; ++code)
     if (vars[code].hybrid0)
@@ -697,8 +690,8 @@ after_setLevel(AfterControl &globs)
   int numplevelDefault = sizeof(plevelDefault) / sizeof(plevelDefault[0]);
   int numhlevelDefault = sizeof(hlevelDefault) / sizeof(hlevelDefault[0]);
 
-  if (iVertID != -1)
-    if (zaxisInqType(iVertID) == ZAXIS_HYBRID && globs.Type > 20) Lhybrid2pressure = true;
+  int iLevelType = (iVertID != -1) ? zaxisInqType(iVertID) : -1;
+  if (iLevelType == ZAXIS_HYBRID && globs.Type > 20) Lhybrid2pressure = true;
 
   if (globs.Verbose) lprintf(stdout);
 
@@ -733,13 +726,15 @@ after_setLevel(AfterControl &globs)
             {
               if (globs.Verbose)
                 {
-                  if (zaxisInqType(iVertID) == ZAXIS_HYBRID)
+                  if (iLevelType == ZAXIS_HYBRID)
                     fprintf(stdout, " All detected hybrid level selected:\n");
                   else
                     fprintf(stdout, " All detected pressure level selected:\n");
                 }
               globs.NumLevelRequest = globs.NumLevelFound;
               for (int l = 0; l < globs.NumLevelRequest; ++l) globs.LevelRequest[l] = LevelFound[l];
+              int maxLev = globs.NumLevelFound;
+              if (maxLev == (globs.numHalfLevels - 1)) globs.LevelRequest[maxLev] = globs.numHalfLevels;
               oVertID = iVertID;
             }
         }
@@ -752,7 +747,7 @@ after_setLevel(AfterControl &globs)
           if (globs.Verbose) fprintf(stdout, " No level detected\n");
           checkLevel = false;
         }
-      else if (globs.NumLevelRequest == 1 && IS_EQUAL(globs.LevelRequest[0], 0))
+      else if (globs.NumLevelRequest == 1 && is_equal(globs.LevelRequest[0], 0))
         {
           if (globs.Verbose) fprintf(stdout, " No level selected\n");
           globs.NumLevelRequest = 0;
@@ -760,24 +755,13 @@ after_setLevel(AfterControl &globs)
         }
       else if (globs.Verbose)
         {
-          if (Lhybrid2pressure)
-            {
-              if (globs.unitsel == 0)
-                fprintf(stdout, " Selected pressure level:\n");
-              else
-                fprintf(stdout, " Selected height level:\n");
-            }
+          if (Lhybrid2pressure) { fprintf(stdout, " Selected %s level:\n", (globs.unitsel == 0) ? "pressure" : "height"); }
           else
             {
-              if (zaxisInqType(iVertID) == ZAXIS_HYBRID)
+              if (iLevelType == ZAXIS_HYBRID)
                 fprintf(stdout, " Selected hybrid level:\n");
               else
-                {
-                  if (globs.unitsel == 0)
-                    fprintf(stdout, " Selected pressure level:\n");
-                  else
-                    fprintf(stdout, " Selected height level:\n");
-                }
+                fprintf(stdout, " Selected %s level:\n", (globs.unitsel == 0) ? "pressure" : "height");
             }
         }
     }
@@ -792,7 +776,7 @@ after_setLevel(AfterControl &globs)
         {
           level = globs.LevelRequest[k];
           for (int l = k + 1; l < globs.NumLevelRequest; ++l)
-            if (removeLevel[l] == false && IS_EQUAL(level, globs.LevelRequest[l]))
+            if (removeLevel[l] == false && is_equal(level, globs.LevelRequest[l]))
               {
                 if (globs.Verbose) fprintf(stdout, "  Level %2d = %13.4f double request\n", l + 1, globs.LevelRequest[l]);
                 removeLevel[l] = true;
@@ -815,7 +799,7 @@ after_setLevel(AfterControl &globs)
               level = globs.LevelRequest[k];
               found = false;
               for (int l = 0; l < globs.NumLevelFound; ++l)
-                if (IS_EQUAL(level, LevelFound[l])) found = true;
+                if (is_equal(level, LevelFound[l])) found = true;
 
               if (!found)
                 {
@@ -857,11 +841,12 @@ after_defineLevel(const AfterControl &globs, struct Variable *vars)
       {
         if (iVertID == -1) break;
 
-        if (zaxisInqType(iVertID) == ZAXIS_HYBRID)
+        int iVertType = zaxisInqType(iVertID);
+        if (iVertType == ZAXIS_HYBRID || iVertType == ZAXIS_HYBRID_HALF)
           {
             if (oVertID == -1)
               {
-                if (globs.NumLevelRequest > globs.NumLevelFound) afterAbort("Too much level requested");
+                if (globs.NumLevelRequest > globs.NumLevelFound) afterAbort("Too many level requested");
 
                 if (globs.NumLevelFound == globs.NumLevelRequest)
                   {
@@ -876,7 +861,17 @@ after_defineLevel(const AfterControl &globs, struct Variable *vars)
                   {
                     oVertID = zaxisCreate(ZAXIS_HYBRID, globs.NumLevelRequest);
                     zaxisDefLevels(oVertID, globs.LevelRequest);
-                    zaxisDefVct(oVertID, globs.nvct, globs.vct);
+                    zaxisDefVct(oVertID, globs.nvct, globs.vct.data());
+                  }
+              }
+
+            if (vars[GEOPOTHEIGHT].selected && oVertID_half == -1)
+              {
+                if (oVertID_half == -1 && globs.NumLevelRequest > 0)
+                  {
+                    oVertID_half = zaxisCreate(ZAXIS_HYBRID_HALF, globs.NumLevelRequest + 1);
+                    zaxisDefLevels(oVertID_half, globs.LevelRequest);
+                    zaxisDefVct(oVertID_half, globs.nvct, globs.vct.data());
                   }
               }
 
@@ -884,10 +879,16 @@ after_defineLevel(const AfterControl &globs, struct Variable *vars)
               {
                 if (vars[code].selected)
                   {
-                    if (vars[code].izaxisID != -1)
-                      if (zaxisInqType(vars[code].izaxisID) == ZAXIS_HYBRID
-                          && zaxisInqSize(vars[code].izaxisID) >= globs.NumLevelRequest)
-                        vars[code].ozaxisID = oVertID;
+                    int zaxisID = vars[code].izaxisID;
+                    if (zaxisID != -1)
+                      {
+                        int zaxisType = zaxisInqType(zaxisID);
+                        int numLevels = zaxisInqSize(zaxisID);
+                        if ((zaxisType == ZAXIS_HYBRID || zaxisType == ZAXIS_HYBRID_HALF) && numLevels >= globs.NumLevelRequest)
+                          {
+                            vars[code].ozaxisID = (code == GEOPOTHEIGHT) ? oVertID_half : oVertID;
+                          }
+                      }
                   }
               }
           }
@@ -918,11 +919,13 @@ after_defineLevel(const AfterControl &globs, struct Variable *vars)
           {
             if (vars[code].selected)
               {
-                if (vars[code].izaxisID != -1)
+                int zaxisID = vars[code].izaxisID;
+                if (zaxisID != -1)
                   {
-                    int nlev = zaxisInqSize(vars[code].izaxisID);
-                    if (zaxisInqType(vars[code].izaxisID) == zaxisInqType(iVertID)
-                        && (nlev == globs.NumLevel || nlev == globs.NumLevel + 1) && nlev > 1)
+                    int zaxisType = zaxisInqType(zaxisID);
+                    int numLevels = zaxisInqSize(zaxisID);
+                    if (zaxisType == zaxisInqType(iVertID) && (numLevels == globs.NumLevel || numLevels == globs.NumLevel + 1)
+                        && numLevels > 1)
                       vars[code].ozaxisID = oVertID;
                   }
               }
@@ -1137,6 +1140,8 @@ after_parini(AfterControl &globs, struct Variable *vars)
     }
   namelist[i] = 0;
 
+  globs.Debug = scan_par(globs.Verbose, namelist, "debug", 0);
+
   if (globs.Debug)
     {
       lprintf(stderr);
@@ -1204,7 +1209,7 @@ after_parini(AfterControl &globs, struct Variable *vars)
 
   scan_darray(namelist, "level", globs.LevelRequest, MaxLevel, &globs.NumLevelRequest);
   if (globs.NumLevelRequest == 1)
-    if (IS_EQUAL(globs.LevelRequest[0], -1)) globs.NumLevelRequest = 0;
+    if (is_equal(globs.LevelRequest[0], -1)) globs.NumLevelRequest = 0;
 
   if (globs.Verbose) lprintf(stdout);
 
@@ -1260,11 +1265,11 @@ after_precntl(AfterControl &globs, struct Variable *vars)
 {
   int vertfound = 0;
   int nhzaxis = 0;
-  int FieldDim = 0;
+  int gridSizeMax = 0;
 
   auto nvars = vlistNvars(globs.ivlistID);
-  auto ngrids = vlistNgrids(globs.ivlistID);
-  auto nverts = vlistNzaxis(globs.ivlistID);
+  auto ngrids = vlistNumGrids(globs.ivlistID);
+  auto nverts = vlistNumZaxis(globs.ivlistID);
   auto ntsteps = vlistNtsteps(globs.ivlistID);
 
   if (globs.Debug)
@@ -1281,7 +1286,7 @@ after_precntl(AfterControl &globs, struct Variable *vars)
       auto gridtype = gridInqType(gridID);
       int datasize = gridInqSize(gridID);
 
-      if (datasize > FieldDim) FieldDim = datasize;
+      if (datasize > gridSizeMax) gridSizeMax = datasize;
 
       if (gridtype == GRID_SPECTRAL && globs.Truncation == 0)
         {
@@ -1314,6 +1319,7 @@ after_precntl(AfterControl &globs, struct Variable *vars)
             case 64: globs.Truncation = 42; break;
             case 48: globs.Truncation = 31; break;
             case 32: globs.Truncation = 21; break;
+            case 1: break;
             default: fprintf(stderr, "%d Gaussian latitudes not supported.\n", globs.Latitudes);
             }
         }
@@ -1324,18 +1330,15 @@ after_precntl(AfterControl &globs, struct Variable *vars)
       auto zaxisID = vlistZaxis(globs.ivlistID, index);
       auto leveltype = zaxisInqType(zaxisID);
       auto numlevel = zaxisInqSize(zaxisID);
-      /*
-        printf("leveltype : %d %d\n", leveltype, zaxisInqSize(zaxisID));
-      */
       if (numlevel > 1)
         {
-          if (leveltype == ZAXIS_HYBRID || leveltype == ZAXIS_PRESSURE)
+          if (leveltype == ZAXIS_HYBRID || leveltype == ZAXIS_HYBRID_HALF || leveltype == ZAXIS_PRESSURE)
             {
-              if (leveltype == ZAXIS_HYBRID && globs.nvct == 0)
+              if ((leveltype == ZAXIS_HYBRID || leveltype == ZAXIS_HYBRID_HALF) && globs.nvct == 0)
                 {
                   nhzaxis++;
                   int nvct = zaxisInqVctSize(zaxisID);
-                  if (numlevel != (nvct / 2 - 1))
+                  if (numlevel != (nvct / 2 - 1) && numlevel != (nvct / 2))
                     {
                       if (nvct == 0)
                         {
@@ -1345,8 +1348,10 @@ after_precntl(AfterControl &globs, struct Variable *vars)
                       continue;
                     }
                 }
-              else if (leveltype == ZAXIS_HYBRID && globs.nvct == zaxisInqVctSize(zaxisID))
-                continue;
+              else if ((leveltype == ZAXIS_HYBRID || leveltype == ZAXIS_HYBRID_HALF) && globs.nvct == zaxisInqVctSize(zaxisID))
+                {
+                  continue;
+                }
 
               if (iVertID != -1) cdo_warning("More than %d different vertical grid structure found!", vertfound);
 
@@ -1359,29 +1364,32 @@ after_precntl(AfterControl &globs, struct Variable *vars)
               LevelFound.resize(globs.NumLevelFound);
               for (int l = 0; l < globs.NumLevelFound; ++l) LevelFound[l] = (int) zaxisInqLevel(zaxisID, l);
 
-              if (leveltype == ZAXIS_HYBRID)
+              if (leveltype == ZAXIS_HYBRID || leveltype == ZAXIS_HYBRID_HALF)
                 {
                   if (globs.nvct == 0)
                     {
                       if (zaxisInqVctSize(zaxisID))
                         {
                           globs.nvct = zaxisInqVctSize(zaxisID);
-
-                          if (globs.vct == nullptr)
+                          globs.numHalfLevels = globs.nvct / 2;
+                          if ((int) globs.vct.size() != globs.nvct)
                             {
-                              globs.vct = (double *) Malloc(globs.nvct * sizeof(double));
-                              array_copy(globs.nvct, zaxisInqVctPtr(zaxisID), globs.vct);
+                              globs.vct.resize(globs.nvct);
+                              zaxisInqVct(zaxisID, globs.vct.data());
                             }
                         }
                       else { afterAbort("VCT not defined in inputfile!"); }
                     }
 
-                  if (numlevel != (globs.nvct / 2 - 1))
-                    afterAbort("Number of hybrid levels %d does not match VCT levels %d", numlevel, globs.nvct / 2 - 1);
+                  if (numlevel == globs.numHalfLevels) { globs.NumLevelFound--; }
+                  else if (numlevel != (globs.numHalfLevels - 1))
+                    {
+                      afterAbort("Number of hybrid levels %d does not match VCT levels %d", numlevel, globs.numHalfLevels - 1);
+                    }
 
                   if (globs.Debug)
-                    for (int i = 0; i < globs.nvct / 2; ++i)
-                      fprintf(stderr, " vct: %4d %10.4f %10.4f\n", i, globs.vct[i], globs.vct[i + globs.nvct / 2]);
+                    for (int i = 0; i < globs.numHalfLevels; ++i)
+                      fprintf(stderr, " vct: %4d %10.4f %10.4f\n", i, globs.vct[i], globs.vct[i + globs.numHalfLevels]);
                 }
 
               if (leveltype == ZAXIS_PRESSURE) globs.AnalysisData = true;
@@ -1424,7 +1432,7 @@ after_precntl(AfterControl &globs, struct Variable *vars)
         {
           if (modelInqNamePtr(modelID))
             {
-              if (std::strncmp(modelInqNamePtr(modelID), "ECHAM5", 6) == 0) Source = S_ECHAM5;
+              if (std::strncmp(modelInqNamePtr(modelID), "ECHAM5", 6) == 0) Source = ECHAM5_Source;
               fprintf(stdout, "%s\n", modelInqNamePtr(modelID));
             }
           else
@@ -1434,8 +1442,8 @@ after_precntl(AfterControl &globs, struct Variable *vars)
 
   for (int varID = 0; varID < nvars; ++varID)
     {
-      int gridID, zaxisID, timeID;
-      vlistInqVar(globs.ivlistID, varID, &gridID, &zaxisID, &timeID);
+      int gridID, zaxisID, timeType;
+      vlistInqVar(globs.ivlistID, varID, &gridID, &zaxisID, &timeType);
       auto code = vlistInqVarCode(globs.ivlistID, varID);
       if (code <= 0 || code >= MaxCodes)
         {
@@ -1458,9 +1466,9 @@ after_precntl(AfterControl &globs, struct Variable *vars)
         fprintf(stderr, "Code %3d  Levels = %3d  LevelType = %3d  GridType = %3d\n", code, numlevel, leveltype, gridtype);
     }
 
-  if (globs.Debug) fprintf(stderr, "FieldDim = %d\n", FieldDim);
+  if (globs.Debug) fprintf(stderr, "FieldDim = %d\n", gridSizeMax);
 
-  globs.Field = (double *) Malloc(FieldDim * sizeof(double));
+  globs.varray.resize(gridSizeMax);
 
   if (globs.Debug)
     for (int code = 0; code < MaxCodes; ++code)
@@ -1606,7 +1614,8 @@ after_readVct(AfterControl &globs, const char *vctfile)
     }
 
   globs.nvct = nlines * 2;
-  globs.vct = (double *) Malloc(globs.nvct * sizeof(double));
+  globs.numHalfLevels = globs.nvct / 2;
+  globs.vct.resize(globs.nvct);
 
   rewind(fp);
 
@@ -1618,10 +1627,10 @@ after_readVct(AfterControl &globs, const char *vctfile)
       double va, vb;
       std::sscanf(line, "%d %lg %lg", &n, &va, &vb);
       globs.vct[i] = va;
-      globs.vct[i + globs.nvct / 2] = vb;
+      globs.vct[i + globs.numHalfLevels] = vb;
       i++;
     }
-  fprintf(stdout, "  Read VCT with %d hybrid levels from file %s\n", globs.nvct / 2 - 1, vctfile);
+  fprintf(stdout, "  Read VCT with %d hybrid levels from file %s\n", globs.numHalfLevels - 1, vctfile);
 
   std::fclose(fp);
 }
@@ -1683,7 +1692,7 @@ after_processing(AfterControl &globs, struct Variable *vars)
             exit(1);
           }
 
-        if (!globs.unitsel && globs.Type >= 20 && globs.NumLevelRequest > 1 && IS_EQUAL(globs.LevelRequest[i], 0))
+        if (!globs.unitsel && globs.Type >= 20 && globs.NumLevelRequest > 1 && is_equal(globs.LevelRequest[i], 0))
           {
             fprintf(stderr, "\n Level %9.2f illegal for Type %d\n", globs.LevelRequest[i], globs.Type);
             exit(1);
@@ -1694,9 +1703,9 @@ after_processing(AfterControl &globs, struct Variable *vars)
 
   after_dimcalc(globs);
 
-  globs.rcoslat = (double *) Malloc(globs.Latitudes * sizeof(double));
-  globs.coslat = (double *) Malloc(globs.Latitudes * sizeof(double));
-  globs.DerivationFactor = (double *) Malloc(globs.Latitudes * sizeof(double));
+  globs.rcoslat.resize(globs.Latitudes);
+  globs.coslat.resize(globs.Latitudes);
+  globs.derivationFactor.resize(globs.Latitudes);
 
   if (globs.Type < 50 && globs.AnalysisData)
     {
@@ -1743,12 +1752,12 @@ after_processing(AfterControl &globs, struct Variable *vars)
       vars[GEOPOTENTIAL].needed |= globs.Type >= 30 || vars[SLP].comp || vars[GEOPOTHEIGHT].comp;
     }
 
-  /*  if ( vars[U_WIND].needed || vars[V_WIND].needed ) */
+  // if ( vars[U_WIND].needed || vars[V_WIND].needed )
   if (vars[U_WIND].comp || vars[V_WIND].comp)
     {
-      globs.dv2uv_f1 = (double *) Malloc(globs.DimSP_half * sizeof(double));
-      globs.dv2uv_f2 = (double *) Malloc(globs.DimSP_half * sizeof(double));
-      geninx(globs.Truncation, globs.dv2uv_f1, globs.dv2uv_f2);
+      globs.dv2uv_f1.resize(globs.DimSP_half);
+      globs.dv2uv_f2.resize(globs.DimSP_half);
+      geninx(globs.Truncation, globs.dv2uv_f1.data(), globs.dv2uv_f2.data());
     }
 
   /* --------- */
@@ -1770,17 +1779,27 @@ after_processing(AfterControl &globs, struct Variable *vars)
   streamClose(globs.istreamID);
 }
 
-class ModuleAfterburner
+class Afterburner : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Afterburner",
+    .operators = { { "after", AfterburnerHelp } },
+    .aliases = { { "afterburner", "after" } },
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { -1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Afterburner> registration = RegisterEntry<Afterburner>(module);
+
   AfterControl globs = {};
   struct Variable vars[MaxCodes + 5];
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
     lstdout = !Options::silentMode;
 
     globs.Verbose = Options::cdoVerbose;
@@ -1805,36 +1824,21 @@ public:
       {
         if (globs.Multi > 0) afterAbort("Namelist parameter MULTI works only with one inputfile");
 
-        ifiles = (const char **) Malloc(globs.Nfiles * sizeof(char *));
+        ifiles.resize(globs.Nfiles);
         for (int i = 0; i < globs.Nfiles; ++i) ifiles[i] = cdo_get_stream_name(--nfiles);
         for (int i = 0; i < globs.Nfiles; ++i) printf("files %d %s\n", i + 1, ifiles[i]);
       }
   }
 
   void
-  run()
+  run() override
   {
     after_processing(globs, vars);
   }
 
   void
-  close()
+  close() override
   {
     FreeMean(vars);
-
-    free(ifile);
-
-    cdo_finish();
   }
 };
-
-void *
-Afterburner(void *process)
-{
-  ModuleAfterburner afterburner;
-  afterburner.init(process);
-  afterburner.run();
-  afterburner.close();
-
-  return nullptr;
-}
