@@ -66,7 +66,7 @@ int
 cdo_stream_inq_timestep(CdoStreamID p_pstreamptr, int tsID)
 {
   Debug(PROCESS_INT, "%s pstreamID %d", p_pstreamptr->m_name, p_pstreamptr->get_id());
-  const auto nrecs = p_pstreamptr->inq_timestep(tsID);
+  auto nrecs = p_pstreamptr->inq_timestep(tsID);
   if (nrecs && tsID == p_pstreamptr->getTsID())
     {
       localProcess->ntimesteps++;
@@ -217,7 +217,7 @@ cdo_open_read(int inStreamIDX)
   if (localProcess->get_stream_cnt_in() < inStreamIDX || inStreamIDX < 0)
     cdo_abort("instream %d of process %d not found", inStreamIDX, localProcess->m_ID);
 
-  const auto inStream = localProcess->inputStreams[inStreamIDX];
+  auto inStream = localProcess->inputStreams[inStreamIDX];
   inStream->open_read();
 
   return inStream;
@@ -369,7 +369,7 @@ cdo_stream_inq_vlist(CdoStreamID p_pstreamPtr)
   if (p_pstreamPtr == nullptr) return -1;
 
   Debug(PROCESS_INT, "Inquiring Vlist from pstream %d", p_pstreamPtr->get_id());
-  const auto vlistID = p_pstreamPtr->inq_vlist();
+  auto vlistID = p_pstreamPtr->inq_vlist();
   if (vlistNumber(vlistID) == CDI_COMP && cdo_stream_number() == CDI_REAL)
     cdo_abort("Fields with complex numbers are not supported by this operator!");
 
@@ -489,6 +489,14 @@ cdo_copy_record(CdoStreamID pstreamPtrDest, CdoStreamID pstreamPtrSrc)
 
 // - - - - - - -
 
+std::pair<int, int>
+cdo_inq_record(CdoStreamID pstreamptr)
+{
+  int varID = -1, levelID = -1;
+  pstreamptr->inq_record(&varID, &levelID);
+  return std::make_pair(varID, levelID);
+}
+
 void
 cdo_inq_record(CdoStreamID pstreamptr, int *varID, int *levelID)
 {
@@ -524,6 +532,6 @@ cdo_set_nan(double missval, size_t gridsize, double *array)
     {
       constexpr double newmissval = -9.e33;
       for (size_t i = 0; i < gridsize; ++i)
-        if (DBL_IS_EQUAL(array[i], missval)) array[i] = newmissval;
+        if (fp_is_equal(array[i], missval)) array[i] = newmissval;
     }
 }
